package agentland.behavior;

import metaglue.*;
import java.rmi.*;
import agentland.resource.*;
import agentland.util.*;
import agentland.behavior.testing.*;

public class BehaviorTesterAgent extends ManagedAgent implements BehaviorTester {

    // class to test behavior stuff

    private BehaviorCoordinator bevCoord;
    private DefaultBehavior defBev;
    private DefaultTestingBehavior defTestBev;
    private OccupiedBehavior occBev;
    private MovieBehavior movieBev;
    private MeetingBehavior meetBev;
    private QuietBehavior qBev;

    public BehaviorTesterAgent() throws RemoteException {
	super();

	bevCoord = (BehaviorCoordinator)reliesOn("agentland.behavior.BehaviorCoordinator");

	// by now bevCoord should have created defBev
	defBev = (DefaultBehavior)reliesOn("agentland.behavior.DefaultBehavior");
	
	// for testing, we have our own DefaultTestingBehavior that is a child of DefaultBehavior
	defTestBev = (DefaultTestingBehavior)reliesOn("agentland.behavior.testing.DefaultTestingBehavior");
	defTestBev.addParent(defBev.getID());
	bevCoord.addBev(defTestBev.getID());
	
	// get other bevs
	occBev = (OccupiedBehavior)reliesOn("agentland.behavior.testing.OccupiedBehavior");
	movieBev = (MovieBehavior)reliesOn("agentland.behavior.testing.MovieBehavior");
	meetBev = (MeetingBehavior)reliesOn("agentland.behavior.testing.MeetingBehavior");
	qBev = (QuietBehavior)reliesOn("agentland.behavior.testing.QuietBehavior");	
    }

    public void sendTestingStart() throws RemoteException {
	notify("perception.testing.start", "");
    }

    public void sendEnter() throws RemoteException {
	notify("perception.enter", "");
    }

    public void sendExit() throws RemoteException {
	notify("perception.exit", "");
    }

    public void sendAllExit() throws RemoteException {
	notify("perception.all.exit", "");
    }

    public void sendPhonecall() throws RemoteException {
	notify("perception.phonecall", "");
    }

    public void sendDvdOn() throws RemoteException {
	notify("perception.dvd.On", "");
    }

    public void sendDvdOff() throws RemoteException {
	notify("perception.dvd.Off", "");
    }

    public void sendMeetingStart() throws RemoteException {
	notify("perception.meeting.Start","");
    }

    public void sendMeetingStop() throws RemoteException {
	notify("perception.meeting.Stop","");
    }

    public void sendQuietOn() throws RemoteException {
	notify("perception.quiet.On","");
    }

    public void sendQuietOff() throws RemoteException {
	notify("perception.quiet.Off","");
    }

    public void sendDoorOpen() throws RemoteException {
	notify("perception.door.Open","");
    }

    public void sendDoorClose() throws RemoteException {
	notify("perception.door.Close","");
    }

    public void sendEvent(String eventName) throws RemoteException {
	notify(eventName, "");
    }

    public void loadOcc() throws RemoteException {
	occBev.addParent(defBev.getID());
	bevCoord.addBev(occBev.getID());
    }

    public void loadMov() throws RemoteException {
	movieBev.addParent(defBev.getID());
	movieBev.addActionDependency(defBev.getID(), "actionLights", "actionLighting");
	bevCoord.addBev(movieBev.getID());
    }

    public void loadBevs() throws RemoteException {
	notify("perception.testing.start", "");

	// establish child-parent connections
	occBev.addParent(defTestBev.getID());
	movieBev.addParent(occBev.getID());
	movieBev.addParent(meetBev.getID());
	meetBev.addParent(occBev.getID());
	meetBev.addParent(movieBev.getID());
	qBev.addParent(defTestBev.getID());

	// establish action dependencies
	// this ==> parent
	movieBev.addActionDependency(defTestBev.getID(), "actionLights", "actionLighting");
	meetBev.addActionDependency(occBev.getID(), "actionPhone", "actionMessenging");
	qBev.addActionDependency(defTestBev.getID(), "actionEnter", "actionWelcome");
	movieBev.addActionDependency(occBev.getID(), "actionPhone", "actionMessenging");
	movieBev.addActionDependency(meetBev.getID(), "actionPhone", "actionPhone");
	meetBev.addActionDependency(movieBev.getID(), "actionPhone", "actionPhone");
	

	/*
	// establish child-parent connections
	occBev.addParent(defBev.getID());
	movieBev.addParent(occBev.getID());
	movieBev.addParent(meetBev.getID());
	meetBev.addParent(occBev.getID());
	meetBev.addParent(movieBev.getID());
	qBev.addParent(defBev.getID());

	// establish action dependencies
	// this ==> parent
	//	movieBev.addActionDependency(defBev.getID(), "actionLights", "actionLighting");
	meetBev.addActionDependency(occBev.getID(), "actionPhone", "actionMessenging");
	//	qBev.addActionDependency(defBev.getID(), "actionEnter", "actionWelcome");
	movieBev.addActionDependency(occBev.getID(), "actionPhone", "actionMessenging");
	movieBev.addActionDependency(meetBev.getID(), "actionPhone", "actionPhone");
	meetBev.addActionDependency(movieBev.getID(), "actionPhone", "actionPhone");
	*/

	// test removing action dependencies
	meetBev.addActionDependency(movieBev.getID(), "actionPhone", "actionLights");
	meetBev.deleteActionDependency(movieBev.getID(), "actionPhone", "actionLights");
	
	//load non-default behaviors
	bevCoord.addBev(occBev.getID());
	bevCoord.addBev(movieBev.getID());
	bevCoord.addBev(meetBev.getID());
	bevCoord.addBev(qBev.getID());
	
    }
    
    public void getBevs() throws RemoteException {
	//	System.out.println(bevCoord.getBevNames());
	log(bevCoord.getBevNames());
    }
    
    public void getTree() throws RemoteException {
	//	log(bevCoord.getTree());
	log("Sorry, not implemented yet!");
    }

    public void getEvents() throws RemoteException {
	//	System.out.println(bevCoord.getEvents());
	log(bevCoord.getEvents());
    }

    public void getDependencies() {
	try {
	    //	    System.out.println(movieBev.getDependencies(defBev.getID()));
	    log(movieBev.getDependencies(defBev.getID()));
	} catch(RemoteException e) {}
    }


}

