/**
 * Created by Ajay Kulkarni 8.10.2001
 */

package agentland.behavior;

import metaglue.*;
import java.rmi.*;
import java.util.*;
import agentland.util.*;
import agentland.behavior.*;
import agentland.device.light.*;
import agentland.device.display.*;
import agentland.device.drapes.*;

/**
 * First Behavior for the Room! Created as a replacement for the 
 * Presentation mess script (agentland.meta."Presentation.clp").
 *
 * @see BehaviorAgent
 * @see PresentationBehavior
 */
public class PresentationBehaviorAgent extends BehaviorAgent 
    implements PresentationBehavior {
    
    private LightManager lights;
    private Projector projector;
    private Drapes drapes;

    // make the following item false before checking in 
    // (only true for testing in rooms w/out projectors)
    private boolean TEST = true;


    // needs to call super(); before running any other code
    // so that the actions get registered 
    public PresentationBehaviorAgent() throws RemoteException {
	super();
	setLogLevel(LogStream.DEBUG);
	addAlias("Presentation Mode");

	lights = (LightManager) reliesOn("agentland.device.light.LightManager");
	if (!TEST)
	    projector = (Projector) reliesOn("agentland.device.display.Projector","4");
	drapes = (Drapes) reliesOn("agentland.device.drapes.DrapesManager");
	
	addEvent("software.powerpoint.action.start", "activate");
	addEvent("software.powerpoint.action.stop", "actionDeactivate");
	addEvent("perception.cloud.door open", "actionLights");

	// voice recog section
	addGrammar("agentland.behavior.speechin.presentationBehavior");

	// start pres voice recog
	addKeyNotePair("startpres","software.powerpoint.action.start");

	// stop pres voice recog
	addKeyNotePair("stoppres","software.powerpoint.action.stop");
	
    }

    // needs to call makeActive(); when it is activated
    public void activate(Secret secret) {
	if (secret.isA("software.powerpoint.action.start")) {
	    try {
		makeActive(); // tell bev coord that you are now active
		lights.setLevel(35); // dimming lights on start
		if (!TEST) {
		    projector.turnOn(); // setting up the projector
		    projector.inputRGB1();
		}
		drapes.close(); // closing drapes on start
		
		bevSay("DEBUG","Presentation Started");
		log("DEBUG","PresBev is now active");
	    } catch(Exception e) {
		log("ERROR", "Error while trying to activate PresentationBehavior: " + e);
	    }
	}
    }

    // needs to call makeListening(); when it is deactivated
    public void actionDeactivate(Secret secret) {
	if (secret.isA("software.powerpoint.action.stop")) {
	    try {
		lights.turnOn(); // restoring lights on end
		drapes.open(); // opening drapes on end
		makeListening();
		bevSay("DEBUG","Presentation Ended");
		log("DEBUG", "PresBev is now not active");
	    } catch(Exception e) {
		log("ERROR", "Error while trying to deactivate PresentationBehavior: " + e);
	    }
	}
    }

    public void actionLights(Secret secret) {
	if (secret.isA("perception.cloud.door open")) {
	    // do nothing... i.e., DON'T want lights to turn on
	    log("DEBUG", "actionLights");
	}
    }
    
}
