/**
 * Created by Ajay Kulkarni 11.26.01
 */

package agentland.behavior;

import metaglue.*;
import java.rmi.*;
import agentland.util.*;
import agentland.device.*;


public class QuietBehaviorAgent extends BehaviorAgent 
    implements QuietBehavior {

    // make the following item false before checking in 
    // (only true for testing in rooms w/out an audio mux)
    private boolean TEST = true;

    // add internal rep for sound control here
    private AudioMux amux;

    // needs to call super(); before running any other code
    // so that actions are registered
    public QuietBehaviorAgent() throws RemoteException {
	super();
	setLogLevel(LogStream.DEBUG);
	addAlias("Quiet Mode");
	addAlias("Business Hours Quiet Mode");

	if (!TEST)
	    amux = (AudioMux) reliesOn("agentland.device.AudioMux");

	addEvent("perception.cloud.door open","actionDoor");
	addEvent("behavior.quiet.start","activate");

	addEvent("perception.cloud.door closed", "actionDoor");
	addEvent("behavior.quiet.stop","actionDeactivate");

	addEvent("device.sensor.motion.stateChange.on","actionDoor");

	/**** voice recog section ****/
	addGrammar("agentland.behavior.speechin.quietBehavior");

	// start quiet voice recog
	addKeyNotePair("startquiet","behavior.quiet.start");

	// stop quiet voice recog
	addKeyNotePair("stopquiet","behavior.quiet.stop");

	// start quiet door action voice recog
	addKeyNotePair("startquietDoor","behavior.quiet.door.start");

	// stop  quiet door action voice recog
	addKeyNotePair("stopquietDoor", "behavior.quiet.door.stop");
	/****************************/
    }

    
    // needs to call makeActive() when it is activated
    public void activate(Secret secret) {
	if (secret.isA("behavior.quiet.start")) {
	    try {
		makeActive(); // tell bev coord that you are now active
		bevSay("DEBUG","Quiet Behavior Started");
		log("DEBUG","QuietBev is now active");
	    } catch(Exception e) {
		log("ERROR", "Error while trying to activate QuietBehavior: " + e);
	    }
	}
    }
    
    
    // needs to call makeListening(); when it is deactivated
    public void actionDeactivate(Secret secret) {
	if (secret.isA("behavior.quiet.stop")) {
	    try {
		makeListening(); // tell bev coord that you are no longer active
		
		// code to reset sound 
		// (in case someone forgets to "close the door" before 
		// deactivating the bev)
		if (!TEST)  
		    amux.unmuteAll(); 
		
		bevSay("DEBUG","Quiet Behavior Ended");
		log("DEBUG","QuietBev is now not active");
	    } catch(Exception e) {
		log("ERROR", "Error while trying to deactivate QuietBehavior: " + e);
	    }
	}
    }

    
    public void actionDoor(Secret secret) {
	if (secret.isA("perception.cloud.door open") ||
	    (secret.isA("behavior.quiet.door.start"))) {

	    //||
	    //	    ((secret.isA("device.sensor.motion.stateChange.on")) &&
	     //	     (secret.details().getBooleanValue() == true) &&
	     //	     (secret.getSource()
							   
	   
	    try {
		// code to mute sound
		if (!TEST)
		    amux.muteAll();
		log("DEBUG", "actionDoor open");
	    } catch(Exception e) {
		log("ERROR", "actionDoor: " + e);
	    }
	}
	
	else if(secret.isA("perception.cloud.door closed") ||
		(secret.isA("behavior.quiet.door.stop"))){
	    try {
		// code to reset sound 
		if (!TEST)  
		    amux.unmuteAll(); 
		log("DEBUG", "actionDoor closed");
	    } catch(Exception e) {
		log("ERROR", "actionDoor: " + e);
	    }
	}
    }
    
    
}
