package agentland.behavior;

import metaglue.*;
import java.rmi.*;
import agentland.resource.*;
import agentland.behavior.speechin.*;
import agentland.gui.*;
import agentland.util.*;
import agentland.behavior.behaviors.*;

/**
 * Loads the basic behaviors into the Room's Behavior Coordinator.
 *
 * @see ManagedAgent
 * @see ReBa
 */
public class ReBaAgent extends ManagedAgent implements ReBa {
    
    private String desig;
    private BehaviorCoordinator bevCoord; 

    /*
    private DefaultBehavior defBev; 
    private PresentationBehavior presBev; 
    private QuietBehavior qBev; 
    private MapSystemBehavior mapBev; 
    private VacantBehavior vacBev;  
    */

    //    private BehaviorSpeech bevspeech;

    // gui stuff
    protected GuiManager gm;

    public ReBaAgent() throws RemoteException {
	super();

	bevCoord = (BehaviorCoordinator)reliesOn("agentland.behavior.BehaviorCoordinator");   

	// if designation is specified, pull up preset configuration
	// otherwise, only default behavior is registered
	desig = (String) getDesignation();
	if (desig.equals("e21")) 
	    configureE21();
	else if (desig.equals("office804"))
	    configureOffice804();
	else if (desig.equals("testing"))
	    configureTesting();

	// gui stuff
	//	gm = (GuiManager)reliesOn("agentland.gui.GuiManager", getDesignation());
	gm = (GuiManager)reliesOn("agentland.gui.GuiManager", "");
	//	gm.addGui(new BCPGuiMaker("Behavior Control Panel", getAgentID()));
	getGui();

	log("DEBUG", "Done Booting...");
    }
    
    public void getGui() throws RemoteException {
	gm.addGui(new BCPGuiMaker("Behavior Control Panel", getAgentID()));
    }

    public void updateGui() throws RemoteException {
	gm.tellGui(new Secret(getAgentID(), "behavior.gui.update"));
    }

    
    private void configureE21() throws RemoteException{
	/*
	//	bevCoord = (BehaviorCoordinator)reliesOn("agentland.behavior.BehaviorCoordinator");   
	defBev = (DefaultBehavior) reliesOn("agentland.behavior.DefaultBehavior");  
	presBev = (PresentationBehavior) reliesOn("agentland.behavior.PresentationBehavior");  
	qBev = (QuietBehavior) reliesOn("agentland.behavior.QuietBehavior");  
	mapBev = (MapSystemBehavior) reliesOn("agentland.behavior.MapSystemBehavior");  
	vacBev = (VacantBehavior) reliesOn("agentland.behavior.VacantBehavior");  
	//	bevspeech = (BehaviorSpeech) reliesOn("agentland.behavior.speechin.BehaviorSpeech"); 

	// bevs that default can transition to
	// 
	// OR
	// 
	// bevs that are listening (i.e., can be activated) when default is active
	// note that they might even be active!
	// ???
	vacBev.addParent(defBev.getID());  
	presBev.addParent(vacBev.getID());  
	qBev.addParent(vacBev.getID());  
	mapBev.addParent(vacBev.getID()); 

	// establish action dependencies
	// this ==> parent
	presBev.addActionDependency(vacBev.getID(), "actionLights", "actionLights");  
		
	
	// adding bevs to ReBa's behavior coordinator
	bevCoord.addBev(vacBev.getID());  
	bevCoord.addBev(presBev.getID()); 
	bevCoord.addBev(qBev.getID());  
	bevCoord.addBev(mapBev.getID());  
	*/


	DefaultBehavior defBev = (DefaultBehavior) reliesOn("agentland.behavior.DefaultBehavior");  
	PresentationBehavior presBev = (PresentationBehavior) reliesOn("agentland.behavior.PresentationBehavior");  
	QuietBehavior qBev = (QuietBehavior) reliesOn("agentland.behavior.QuietBehavior");  
	MapSystemBehavior mapBev = (MapSystemBehavior) reliesOn("agentland.behavior.MapSystemBehavior");  
	VacantBehavior vacBev = (VacantBehavior) reliesOn("agentland.behavior.VacantBehavior");  

	// establish transitional relationships
	vacBev.addParent(defBev.getID());  
	presBev.addParent(vacBev.getID());  
	qBev.addParent(vacBev.getID());  
	mapBev.addParent(vacBev.getID()); 

	// establish action dependencies
	// this ==> parent
	presBev.addActionDependency(vacBev.getID(), "actionLights", "actionLights");  

	// adding bevs to the behavior coordinator
	bevCoord.addBev(vacBev.getID());  
	bevCoord.addBev(presBev.getID()); 
	bevCoord.addBev(qBev.getID());  
	bevCoord.addBev(mapBev.getID());  
    }

    private void configureOffice804() throws RemoteException {
	DefaultBehavior defBev = (DefaultBehavior) reliesOn("agentland.behavior.DefaultBehavior");  
	VacantBehavior vacBev = (VacantBehavior) reliesOn("agentland.behavior.VacantBehavior");
	Office804WelcomeBehavior welBev = (Office804WelcomeBehavior) reliesOn("agentland.behavior.behaviors.Office804WelcomeBehavior");

	// establish transitional relationships
	vacBev.addParent(defBev.getID());
	welBev.addParent(vacBev.getID());
	
	// establish action dependencies
	// this ==> parent
	welBev.addActionDependency(vacBev.getID(), "actionLights", "actionLights");

	// adding bevs to the behavior coordinator
	bevCoord.addBev(vacBev.getID());
	bevCoord.addBev(welBev.getID());
    }

    private void configureTesting() throws RemoteException {
	log("nothing");
    }
			
}

	




