package agentland.behavior.speechin;

import java.rmi.*;
import java.util.*;
import speech.*;
import speech.tools.*;
import metaglue.*;
import agentland.util.asker.*;
import agentland.behavior.*;

/**
 * Speech Agent for Behaviors. 
 *
 * @see ManagedSpeechAgent
 * @see Test
 */
public class BehaviorSpeechAgent extends ManagedSpeechAgent implements BehaviorSpeech {

    private Asker ask = null;
    private BehaviorCoordinator bevcoord; 
    private Hashtable keyToNoteHash;
    private Hashtable keyToSpokenMsgHash;

    // flag that determines whether DEBUG speech messages are spoken 
    private Attribute debugSpeech = new Attribute("debugSpeech");

    public BehaviorSpeechAgent() throws RemoteException {
	keyToNoteHash = new Hashtable();
	keyToSpokenMsgHash = new Hashtable();

	ask = (Asker) requestAgent("speech.tools.Asker");

	AppGrammar g = loadGrammar("agentland.behavior.speechin.Behavior");
	
	say("DEBUG","This is the behavior speech agent. ");
    }


    public void acceptedTagsResult(Hashtable tags) throws RemoteException {

	if (tags.containsKey("regbevs")) {
	    bevcoord = (BehaviorCoordinator) reliesOn("agentland.behavior.BehaviorCoordinator");
	    String bevNames = bevcoord.getBevNames();
	    say(bevNames);
	    //	    log(bevNames);
	    //	    say("The registered behaviors are as follows: \n" + bevNames);
	    //	    log("The registered behaviors are as follows: \n" + bevNames);
	} else if(tags.containsKey("activebevs")) {
	    bevcoord = (BehaviorCoordinator) reliesOn("agentland.behavior.BehaviorCoordinator");
	    String bevNames = bevcoord.getActiveBevNames();
	    say(bevNames);
	    //	    log(bevNames);
	    //	    say("The active behaviors are as follows: \n" + bevNames);
	    //	    log("The active behaviors are as follows: \n" + bevNames);
	} else if(tags.containsKey("listeningbevs")) {
	    bevcoord = (BehaviorCoordinator) reliesOn("agentland.behavior.BehaviorCoordinator");
	    String bevNames = bevcoord.getListeningBevNames();
	    say(bevNames);
	    //	    log(bevNames);
	    //	    say("The listening behaviors are as follows\n" + bevNames);
	    //	    log("The listening behaviors are as follows\n" + bevNames);
	} else if(tags.containsKey("inactivebevs")) {
	    bevcoord = (BehaviorCoordinator) reliesOn("agentland.behavior.BehaviorCoordinator");
	    String bevNames = bevcoord.getInactiveBevNames();
	    say(bevNames);
	    //	    log(bevNames);
	    //	    say("The inactive behaviors are as follows\n" + bevNames);
	    //	    log("The inactive behaviors are as follows\n" + bevNames);
	}		    

	else { 
	    
	    String note = lookupNote((String)tags.get("rule"));
	    String spokenMsg = lookupSpokenMsg((String)tags.get("rule"));

	    //	    log("note: " + note);
	    //	    log("spokenMsg: " + spokenMsg);
	    
	    if (!note.equals(""))
		notify(note,"");
	    if (!spokenMsg.equals(""))
		say(spokenMsg);
	    if (note.equals("") && spokenMsg.equals(""))
		log("WARNING", "Unrecognized utterance");
	    
	}

    }


    public void addKeyNotePair(String key, String note) throws RemoteException {
	keyToNoteHash.put(key, note);
    }

    public void addKeySpokenMsgPair(String key, String spokenMsg) throws RemoteException {
	keyToSpokenMsgHash.put(key, spokenMsg);
    }
    
    protected String lookupNote(String key) {
	if (keyToNoteHash.containsKey(key))
	    return (String) keyToNoteHash.get(key);
	else 
	    return new String();
    }

    protected String lookupSpokenMsg(String key) {
	if (keyToSpokenMsgHash.containsKey(key))
	    return (String) keyToSpokenMsgHash.get(key);
	else
	    return new String();
    }
    
    public AppGrammar loadBehaviorGrammar(String gram) throws RemoteException {
	return loadGrammar(gram);
    }

    public void say(String s)  {
	super.say(s);
    }

    public void say(String flag, String s) {
	try {
	    if (flag.equalsIgnoreCase("DEBUG")) {
		if (debugSpeech.getBooleanValue())
		    super.say(s);
	    }
	    else {
		log("ERROR", "Error saying \"" + s + "\" with flag \""
		    + flag + "\"");
	    }
	} catch(Exception e) {
	    log("ERROR", "Error saying \"" + s + "\" with flag \": " + e);
	}
    }
}





